
import logging

LOG = logging.getLogger("gm_apidesc")


def _format_with_kwargs(msg_format, kwargs):

    return msg_format % kwargs


class Error(Exception):

    message_format = None

    def __init__(self, message=None, **kwargs):
        try:
            message = self._build_message(message, **kwargs)
        except KeyError:
            message = self.message_format

        super(Error, self).__init__(message)

    def _build_message(self, message, **kwargs):

        if message:
            return message

        return _format_with_kwargs(self.message_format, kwargs)


class ValidateError(Error):

    def __init__(self, message=None):
        super(ValidateError, self).__init__(message)


class VisibleExInvalidInput(ValidateError):
    """参数格式不对"""

    message_format = 'Error: In data: %(input)s, invalid %(field)s: %(expect)s %(error)s'

    def __init__(self, field, value, expect_cls=None, error=''):
        expect = expect_cls.tostring() \
            if expect_cls and hasattr(expect_cls, 'tostring') else ''
        self.dbgmsg = self.message_format % {'field': field,
                                             'input': value,
                                             'expect': expect,
                                             'error': error}
        LOG.error(self.dbgmsg)
        super(VisibleExInvalidInput, self).__init__(
            '%(field)s输入错误：%(error)s' % {'field': field, 'error': error})


class VisibleExMissingArgument(ValidateError):
    """参数缺失"""

    message_format = 'Error: in %(input)s, missing argument %(name)s'

    def __init__(self, argname, value):
        self.dbgmsg = self.message_format % {'name': argname, 'input': value}
        LOG.error(self.dbgmsg)
        super(VisibleExMissingArgument, self).__init__('缺少参数：%s' % argname)


class VisibleExUnknownArgument(ValidateError):
    """未知参数、多余参数"""

    message_format = 'Error: Unknown argument: %(name)s'

    def __init__(self, argname):
        self.dbgmsg = self.message_format % {'name': argname}
        LOG.error(self.dbgmsg)
        super(VisibleExUnknownArgument, self).__init__('不接受参数：%s' % argname)


class NotMatchError(Error):
    """不匹配"""

    message_format = 'Error: Type data: %(input)s, invalid %(field)s: %(expect)s %(error)s'

    def __init__(self, _type, expect=None, error=''):

        super(VisibleExInvalidInput, self).__init__(
            '%(_type)s 匹配错误（%(expect)s）：%(error)s'
            % {'_type': _type, 'error': error, 'expect': expect}
        )


class VisibleExValueError(ValidateError):
    pass
