"""对入口进行装饰。

使用场景:
    1、django http 请求
    2、gm_rpcd
    3、普通函数
"""

import functools
import inspect

from django.views.generic import View
from django.http import HttpRequest

from gm_apidesc import runexc
from gm_apidesc import check
from gm_apidesc.check import MetaObject
from gm_apidesc.types import filter_inner_type
from gm_apidesc.types import Mapper


# schema 的几个参数，以及对应的检测工具
SCHEMA_CHENER = {
    "type": check.register_type,
    'param': check.register_param,
    'return': check.register_return,
    'desc': check.register_desc,
}


class Checker(object):

    def __init__(self, func):
        # FullArgSpec(args=['a', 'b', 'c'], varargs=None, varkw=None,
        #             defaults=(None,), kwonlyargs=[], kwonlydefaults=None, annotations={})
        func_args = inspect.getfullargspec(func)
        self.kwargsname = None
        self._param_names = func_args.args
        self.param_defs = {}
        self.name = func.__name__
        self.desc = None
        self.return_type = None
        self.return_desc = None
        self.params_have_default = []
        self.req = None

        # 获取默认参数
        default_len = len(func_args.defaults) if func_args.defaults else 0
        self.params_have_default = func_args.args[-default_len]

    def add_type_check(self, param_name, param_type, param_desc='', required=True,
                       nullable=False):

        if param_name in self.params_have_default:
            required = False
        param_type = filter_inner_type(param_type)
        if isinstance(param_type, Mapper):
            param_type.set_valuename(param_name)

        self.param_defs[param_name] = MetaObject(type=param_type,
                                                 desc=param_desc,
                                                 required=required,
                                                 nullable=nullable)

    def add_desc(self, desc):
        self.desc = desc

    def add_ret(self, r_type, desc):

        r_type = filter_inner_type(r_type)
        self.return_type = MetaObject(type=r_type, desc=desc)
        self.return_desc = desc

    def get_elem_def(self, param_name):
        if param_name in self.param_defs:
            return self.param_defs[param_name]

    def get_req_args(self, args, kwargs):
        """判断参数中是否有req，并获取req.GET/POST"""

        http_args = {}
        if not args:
            return [], {}

        if isinstance(args[0], View):  # django class view
            args = args[1:]

        if isinstance(args[0], HttpRequest):  # django class view
            request = args[0]
            args = args[1:]
            http_args.update(request.GET.dict())
            http_args.update(request.POST.dict())

        return args, http_args

    @property
    def param_names(self):

        return self._param_names


def schema(pattern):

    api_schema_pattern = pattern

    def decorator(func):

        checker = Checker(func)

        if not isinstance(api_schema_pattern, dict):
            raise runexc.SchemaFormatError("schema必须是字典")

        for key, value in api_schema_pattern.items():
            if key not in SCHEMA_CHENER:
                raise runexc.SchemaFormatError()

            SCHEMA_CHENER[key](checker, value)


        @functools.wraps(func)
        def wrapper(*args, **kwargs):

            check.validate_input(checker, args, kwargs)

            result = func(*args, **kwargs)

            # validate_output(return_type, result)

            return result

        return wrapper

    return decorator

